﻿using System;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using uFR;

namespace ufr_access_control
{
    public partial class frmMain : Form
    {
        string assemblyVersion = System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString();
        private Thread instanceOfThread;
        private Looper instanceOfLooper;
        private Mutex mut;
        private bool mDigitalInState = false;
        private bool mIntercomInState = false;
        private bool mFirstRun = false;
        public delegate void NewCardInFieldDelegate(byte[] uid, byte uid_len);
        public NewCardInFieldDelegate newCardInField;
        public delegate void DigitalInputsDelegate(byte intercom, byte dig_in, byte relay_state);
        public DigitalInputsDelegate digitalInputs;

        private void newCardInFieldHandler(byte[] uid, byte uid_len)
        {
            if (uid_len == 0)
            {
                textBox1.Text = "No card in field";
                textBox1.BackColor = SystemColors.Control;
            }
            else
            {
                textBox1.BackColor = Color.LightGreen;
                textBox1.Text = BitConverter.ToString(uid, 0, (Int32)uid_len);
            }
        }

        private void digitalInputsHandler(byte intercom, byte dig_in, byte relay_state)
        {
            mIntercomInState = intercom != 0;
            mDigitalInState = dig_in != 0;
            chkDigitalIn.Checked = mDigitalInState;
            chkIntercomIn.Checked = mDigitalInState;
            if (relay_state != 0)
            {
                btnRelayOn.BackColor = Color.LightGreen;
                btnRelayOff.BackColor = SystemColors.Control;
            }
            else
            {
                btnRelayOn.BackColor = SystemColors.Control;
                btnRelayOff.BackColor = Color.LightPink;
            }

        }

        public frmMain()
        {
            bool tryDefaultDllPath = false;

            InitializeComponent();

            newCardInField = new NewCardInFieldDelegate(newCardInFieldHandler);
            digitalInputs = new DigitalInputsDelegate(digitalInputsHandler);
#if WIN64
            string DllPath = "..\\..\\..\\lib\\windows\\x86_64"; // for x64 target
#else
            string DllPath = "..\\..\\..\\lib\\windows\\x86"; // for x86 target
#endif
            string path = Path.GetTempPath();
            string assemblyProbeDirectory = DllPath;
            Directory.SetCurrentDirectory(assemblyProbeDirectory);
            try
            {
                readLibVersion();
            }
            catch (System.DllNotFoundException)
            {
                tryDefaultDllPath = true;
            }
            catch (System.BadImageFormatException)
            {
                tryDefaultDllPath = true;
            }
            Directory.SetCurrentDirectory(path);
            if (tryDefaultDllPath)
                readLibVersion();
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            this.Text = this.Text + " v" + assemblyVersion;
        }

        private void frmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (mFirstRun)
            {
                if (btnCloseReader.Enabled)
                {
                    instanceOfLooper.stop();
                    instanceOfThread.Abort();
                    instanceOfThread.Join();
                }
            }
        }

        private void readLibVersion()
        {
            uint lib_ver;
            byte lib_major_ver;
            byte lib_minor_ver;
            ushort lib_build;


            //-------------------------------------------------------
            lib_ver = uFCoder.GetDllVersion();
            lib_major_ver = (byte)lib_ver;
            lib_minor_ver = (byte)(lib_ver >> 8);
            lib_build = (byte)(lib_ver >> 16);

            DevInfoDLL.Text = " DLL: " + (lib_major_ver) + "." + (lib_minor_ver) +
                  "." + (lib_build);
        }

        private void onReaderOpen()
        {
            statusReader.Text = " CONNECTED ";
            panelReader.BackColor = Color.LimeGreen;

            btnOpenReader.Enabled = false;
            btnCloseReader.Enabled = true;
            pnlControls.Enabled = true;

            mut = new Mutex();
            instanceOfLooper = new Looper(this, mut);
            instanceOfThread = new Thread(new ThreadStart(instanceOfLooper.run));
            instanceOfThread.Start();
            mFirstRun = true;
        }

        private void onReaderClose()
        {
            statusReader.Text = " Not connected";

            panelReader.BackColor = Color.Red;

            DevInfoSN.Text = " SN:";
            DevInfoHW.Text = " HW:";
            DevInfoFW.Text = " FW:";

            btnOpenReader.Enabled = true;
            btnCloseReader.Enabled = false;
            pnlControls.Enabled = false;

            instanceOfLooper.stop();
            instanceOfThread.Abort();
            instanceOfThread.Join();
            instanceOfThread = null;
            instanceOfLooper = null;
            mut.Dispose();
            mut = null;
        }

        private void showStatus(DL_STATUS status, string ok_text)
        {
            string msg;
            string status_msg = status.ToString().Replace("UFR_", "").Replace('_', ' ');

            // remove from start UFR_
            // all _ to space

            if (status == DL_STATUS.UFR_OK)
            {
                msg = " OK - " + ok_text;
                statusResult.BackColor = Color.Lime;
            }
            else
            {
                //msg = " Error: " + status_msg;
                msg = " " + status_msg;

                statusResult.BackColor = Color.Red;
            }

            // prn
            statusResult.Text = msg;
        }

        private DL_STATUS openReader()
        {
            DL_STATUS status;

            UInt32 reader_type;
            string reader_sn;
            byte fw_major_ver;
            byte fw_minor_ver;
            byte fw_build;
            byte hw_major;
            byte hw_minor;

            //-------------------------------------------------------
            status = uFCoder.ReaderOpen();
            if (status != DL_STATUS.UFR_OK)
            {
                return status;
            }

            //-------------------------------------------------------
            status = uFCoder.GetReaderSerialDescription(out reader_sn);
            status |= uFCoder.GetReaderType(out reader_type);

            status |= uFCoder.GetReaderHardwareVersion(out hw_major, out hw_minor);
            status |= uFCoder.GetReaderFirmwareVersion(out fw_major_ver, out fw_minor_ver);

            status |= uFCoder.GetBuildNumber(out fw_build);

            if (status != DL_STATUS.UFR_OK)
            {
                return status;
            }

            //-------------------------------------------------------

            DevInfoSN.Text = " SN : " + reader_sn;

            DevInfoHW.Text = " HW : " + (int)hw_major + "." + hw_minor;

            DevInfoFW.Text = " FW : " + (fw_major_ver) + "." +
                                    (fw_minor_ver) + "." +
                                    (fw_build);

            return DL_STATUS.UFR_OK;
        }

        private void bOpenReader_Click(object sender, EventArgs e)
        {
            DL_STATUS status;

            panelReader.BackColor = Color.DarkRed;
            statusResult.Text = "Please wait...";
            this.Update();

            status = openReader();

            showStatus(status, "Reader Open");

            if (status == DL_STATUS.UFR_OK)
            {
                onReaderOpen();
            }
            else
            {
                onReaderClose();
            }
        }

        private void bCloseReader_Click(object sender, EventArgs e)
        {
            uFCoder.ReaderClose();
            onReaderClose();
            statusResult.Text = "";
            statusResult.BackColor = Control.DefaultBackColor;
        }

        private void btnLockOn_Click(object sender, EventArgs e)
        {
            DL_STATUS status;

            mut.WaitOne();
            status = uFCoder.UfrXrcLockOn(300);
            mut.ReleaseMutex();

            showStatus(status, "Lock open");
        }

        private void btnRelayOn_Click(object sender, EventArgs e)
        {
            DL_STATUS status;

            mut.WaitOne();
            status = uFCoder.UfrXrcRelayState(1);
            mut.ReleaseMutex();

            showStatus(status, "Relay activated");
        }

        private void btnRelayOff_Click(object sender, EventArgs e)
        {
            DL_STATUS status;

            mut.WaitOne();
            status = uFCoder.UfrXrcRelayState(0);
            mut.ReleaseMutex();

            showStatus(status, "Relay deactivated");
        }

        private void chkDigitalIn_CheckedChanged(object sender, EventArgs e)
        {
            chkDigitalIn.Checked = mDigitalInState;
        }

        private void chkIntercomIn_CheckedChanged(object sender, EventArgs e)
        {
            chkIntercomIn.Checked = mIntercomInState;
        }

        private void linkLabel_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            ProcessStartInfo sInfo = new ProcessStartInfo("http://www.d-logic.net");
            Process.Start(sInfo);
        }
    }

    public class Looper
    {
        Form form;
        private Mutex mut;
        private bool stop_request = false;
        private IAsyncResult result;

        public Looper(Form param_form, Mutex param_mut)
        {
            form = param_form;
            mut = param_mut;
        }

        public void stop()
        {
            stop_request = true;
        }

        byte[] uid = new byte[] { 1, 2, 3, 4 };

        static bool ByteArrayCompare(byte[] b1, byte[] b2)
        {
            if (b1.Length == b2.Length)
                for (int i = 0; i < b1.Length; i++)
                    if (b1[i] != b2[i])
                        return false;
            else
                return false;
            return  true;
        }

        public void run()
        {
            DL_STATUS status;
            byte prev_intercom = 0, prev_dig_in = 0, prev_relay_state = 0;
            byte intercom, dig_in, relay_state;
            byte[] uid = new byte[10], old_uid = new byte[10];
            byte uid_len, old_uid_len = 0, card_type, old_card_type = 0;

            while (!stop_request)
            {
                mut.WaitOne();
                status = uFCoder.GetCardIdEx(out card_type, uid, out uid_len);
                mut.ReleaseMutex();
                if (status == DL_STATUS.UFR_OK)
                {
                    if (old_card_type != card_type
                        || old_uid_len != uid_len
                        || !ByteArrayCompare(old_uid, uid))
                    {
                        old_card_type = card_type;
                        old_uid_len = uid_len;
                        form.Invoke(((frmMain)form).newCardInField, new Object[] { uid, uid_len });
                    }
                }
                else
                {
                    old_uid_len = uid_len = 0;
                    form.Invoke(((frmMain)form).newCardInField, new Object[] { uid, uid_len });
                }

                mut.WaitOne();
                status = uFCoder.UfrXrcGetIoState(out intercom, out dig_in, out relay_state);
                mut.ReleaseMutex();
                if (status == DL_STATUS.UFR_OK)
                {
                    if (prev_intercom != intercom || prev_dig_in != dig_in || prev_relay_state != relay_state)
                    {
                        prev_intercom = intercom;
                        prev_dig_in = dig_in;
                        prev_relay_state = relay_state;
                        form.Invoke(((frmMain)form).digitalInputs, new Object[] { intercom, dig_in, relay_state });
                    }
                }

                uid[3]++;
                Thread.Sleep(250);
            }
        }
    }
}
